// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _DataStore_h__
#define _DataStore_h__

#include "Identifier.h"
#include "Enums.h"
#include <vector>
using HInput::CombinationOperator::COValue;
#include "Misc.h"
#include <string>

namespace HInput
{
	/*
		ContextData encapsulates the context sensitive data 
		passed from the data store to the input creation factory.
	*/
	struct ContextData
	{
	public:
		ContextData()
			: Context1(0)
			, Context2(0)
			, ContextFloat(0.0f)
		{}

		ContextData(unsigned context1, unsigned context2, float contextFlt)
			: Context1(context1)
			, Context2(context2)
			, ContextFloat(contextFlt)
		{}
		
		unsigned	Context1;
		unsigned	Context2;
		float		ContextFloat;
	};

	/*
		This is a record in the data store
	*/
	class DataRecord
	{
	public:
		DataRecord();
		DataRecord(const Identifier& inputID, const Identifier& setID, const Identifier& typeID, unsigned context1 = 0, unsigned context2 = 0, float contextFlt = 0.0f, CombinationOperator::COValue comboMethod = CombinationOperator::None);
		~DataRecord();
	
		const Identifier&	GetInputSetID	() const { return m_InputSetID; }
		const Identifier&	GetInputID		() const { return m_InputID; }
		const Identifier&	GetInputTypeID	() const { return m_InputTypeID; }
		COValue				GetCombineMethod() const { return m_CombineMethod; }
		const ContextData&	GetContext		() const { return m_Context; }
		
		void SetInputSetID   (const Identifier& val) { m_InputSetID = val; }
		void SetInputID      (const Identifier& val) { m_InputID = val; }
		void SetInputTypeID  (const Identifier& val) { m_InputTypeID = val; }
		void SetCombineMethod(const COValue& val) { m_CombineMethod = val; }
		void SetContext		 (const ContextData& val) { m_Context = val; }

	private:
		Identifier		m_InputSetID;
		Identifier		m_InputID;
		Identifier		m_InputTypeID;
		ContextData		m_Context;
		COValue			m_CombineMethod;
	};

	/*
		This is the data store for the input system.
	*/
	class DataStore : private Misc::Uncopyable
	{
	private:
		friend class InputManager;
		DataStore();

	public:
		~DataStore();
		
		void	AddRecord(const DataRecord& record);
		void	ClearRecords();
		void	Save();
		void	Load();

		// Direct access to records, for iterating etc.
		std::vector<DataRecord>& Records() { return m_Records; }

		// Record helpers
		void	GetInputSets(std::vector<Identifier>& outSets) const;

	private:

		const std::string&	GetDataFilename() const;
		char				GetVersion() const;
		void				LoadDefaultInputs();

		std::vector<DataRecord>	m_Records;
	};


	class DataHelper
	{
	private:
		DataHelper();
		~DataHelper();

	public:
		static void	WriteDataRecord(const DataRecord& record, std::ofstream& file);
		static void	WriteIdentifier(const Identifier& ident, std::ofstream& file);
		static void	WriteContext(const ContextData& context, std::ofstream& file);
		static void	WriteCombineMethod(COValue covalue, std::ofstream& file);

		// These all return the number of bytes read
		static unsigned	ReadDataRecord(DataRecord& outRecord, const char* pBuf);
		static unsigned	ReadIdentifier(Identifier& outIdent, const char* pBuf);
		static unsigned	ReadContext(ContextData& outContext, const char* pBuf);
		static unsigned	ReadCombineMethod(COValue& outCovalue, const char* pBuf);
		
		static void	CheckFileDirExists();
	};

} // end namespace HInput

#endif // _DataStore_h__