// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _Input_h__
#define _Input_h__

#include "Enums.h"
#include "Identifier.h"
#include <list>
#include <string>
#include "SmartPtr.h"

namespace HInput
{
	/* CODE TEMPLATE FOR IMPLEMENTATIONS

	class Input_XXXX : public Input
	{
	public:
	Input_XXXX(PlayerID plyr, const DataRecord& data)
	: Input(plyr, data) {}
	virtual ~Input_XXXX() {}

	// Derived classes must call: SetAvailable(), 
	// SetSatisfied() and SetJustUpdated() in here
	virtual void Update();

	// This need not do anything, but it must be present
	static void PopulateMetaData(InputMetaData& outData);
	};

	*/

	// Fwd decs
	class DataRecord;
	class InputMetaData;
	class InputManager;

	/*
		This is an interface which must be implemented by every 
		class that provides a concrete input implementation, 
		e.g. Input_GamepadButtonPress

		The basic requirements are:
		
		- The derived class MUST implement a public, static, NON-VIRTUAL:
			static void PopulateMetaData(InputMetaData& outData);
		  Inside, it may initialize its metadata, allowing tools to 
		  provide better interfaces.

		- It must indicate whether it was available on the last 
		  frame it received an update.

	    - It must indicate whether the input it represents was
		  'satisfied' on the last frame it received an update.
		  The definition of 'satisfied' is of course dependent
		  on context.

	    - It must indicate the frame sequence number on which
		  it was last updated, and it must keep this up to date
		  by calling SetJustUpdated() in its Update() function.

	    - It could be composed in a tree structure with a sibling 
		  and/or child, so availability and satisfaction may depend 
		  on more than this single instance. GetValue() is not affected.
		  The protected SetAvailable and SetSatisfied() methods will 
		  automatically handle updating and querying the child or sibling 
		  if necessary, so they should always be called, even if there is
		  seemingly no change to the local value.

	    - [Optional] It may choose to provide a float representing
		  some context sensitive data which the client needs (e.g.
		  analogue axis value, time button held, etc). This feature
		  is not compatible with the AND/OR tree and so only the
		  base input will be counted for this purpose.

	    An instance is allowed to store state internally, as it 
		will be created specially for a particular player.
	*/
	class Input
	{
	private:
		Input(); // Hide default c'tor
		Input(const Input&); // Don't allow copying
		const Input& operator=(const Input&); // Don't allow copying

	public:
		Input(PlayerID plyr, const DataRecord& data, InputManager& manager);
		virtual ~Input();

		// Derived classes must implement, and must call: 
		// SetAvailable(), SetSatisfied() and SetJustUpdated()
		// in their Update() method.
		virtual void Update() = 0;
		
		// Public getters
		PlayerID			GetPlayerID()	const { return m_PlayerID; }
		const Identifier&	GetInputID()	const { return m_InputID; }
		const Identifier&	GetInputTypeID()const { return m_InputTypeID; }
		bool				WasAvailable()	const { return m_bWasAvailable; }
		bool				WasSatisfied()	const { return m_bWasSatisfied; }
		float				GetValue()		const { return m_fValue; }
		unsigned			GetFrameLastUpdated() const { return m_FrameLastUpdated; }

		// After these calls, the Input instance takes over memory management on
		// these pointers, which are assumed to be allocated on the heap. NULL
		// pointers are permitted.
		void		SetOrSibling(const shared_ptr<Input>& pSibling);
		void		SetAndChild (const shared_ptr<Input>& pChild);
		void		AppendOrSibling(const shared_ptr<Input>& pSibling);
		void		AppendAndChild (const shared_ptr<Input>& pChild);

	protected:
		// Protected setters (for derived classes)
		void		SetAvailable(bool bVal);
		void		SetSatisfied(bool bVal);
		void		SetValue(float fVal);
		void		SetJustUpdated();

		// Back reference to the active input manager. This is supposed
		// to be a singleton, but using inputs in other dlls breaks that.
		InputManager& m_rManager;

	private:		
		void				CheckUpdated_Child();
		void				CheckUpdated_Sibling();
		shared_ptr<Input>	m_pOrSibling;	// This input is our sibling - use it to represent logical OR
		shared_ptr<Input>	m_pAndChild;	// This input is our child - use it to represent logical AND

		// These values are immutable after construction
		PlayerID	m_PlayerID;
		Identifier	m_InputID;
		Identifier	m_InputTypeID;

		// These values may change each update
		bool		m_bWasAvailable;
		bool		m_bWasSatisfied;
		float		m_fValue;
		unsigned	m_FrameLastUpdated;
	};

} // end namespace HInput

#endif // _Input_h__