// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _InputFactory_h__
#define _InputFactory_h__

#include <vector>
#include "Identifier.h"
#include "Enums.h"
#include "Input.h"
#include "SmartPtr.h"
#include "Misc.h"

namespace HInput
{
	// Fwd decs
	class DataRecord;
	class InputMetaData;
	class InputManager;

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

	/*
		Just a base class allowing templated subclasses
	*/
	class InputCreatorBase
	{
	public:
		InputCreatorBase(){ /**/ }
		virtual ~InputCreatorBase(){ /**/ }
	
		virtual shared_ptr<Input>		Create		(PlayerID plyr, const DataRecord& data, InputManager& manager) = 0;
		virtual const std::type_info&	GetTypeInfo	() = 0;
		virtual void					PopulateMetaData(InputMetaData& outData) = 0;
	};

	/*
		Template allowing creation of arbitrary Input subclasses,
		it is assumed that T is derived from Input, that it
		has a c'tor taking the same two arguments and implements
		a static method named PopulateMetaData which takes a non-const
		reference to an InputMetaData instance
	*/
	template<typename T>
	class InputCreator : public InputCreatorBase
	{
	public:
		InputCreator(){ /**/ }
		~InputCreator(){ /**/ }

		shared_ptr<Input>		Create		(PlayerID plyr, const DataRecord& data, InputManager& manager);
		const std::type_info&	GetTypeInfo	();
		void					PopulateMetaData(InputMetaData& outData);
	};

	//------------------------------------------------------------------
	// PopulateMetaData									[HG: 8/2/2010]
	//------------------------------------------------------------------
	template<typename T>
	void HInput::InputCreator<T>::PopulateMetaData( InputMetaData& outData )
	{
		// This is where we assume that the instructions for writing
		// inputs have been followed, and the class implements the 
		// appropriate static method. Check this first if you have a 
		// compile time error here.
		T::PopulateMetaData(outData);
	}

	//------------------------------------------------------------------
	// InputCreator<T>::GetTypeInfo						[HG: 3/2/2010]
	//------------------------------------------------------------------
	template<typename T>
	const std::type_info& InputCreator<T>::GetTypeInfo()
	{
		return typeid(T);
	}

	//------------------------------------------------------------------
	// InputCreator<T>::Create							[HG: 3/2/2010]
	//------------------------------------------------------------------
	template<typename T>
	shared_ptr<Input> InputCreator<T>::Create( PlayerID plyr, const DataRecord& data, InputManager& manager )
	{
		return shared_ptr<Input>( static_cast<Input*>( new T(plyr, data, manager) ) );
	}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

	/*
		This class maintains a register of classes to input
		type identifiers, and satisfies requests to create
		input instances
	*/
	class InputFactory : private Misc::Uncopyable
	{
	private:
		friend class InputManager;
		InputFactory();

	public:
		~InputFactory();
		
		// Static templated helper for registering input implementations
		//template<typename T>
		//static void RegisterInput(const Identifier& id)
		//{
		//	InputCreatorBase* pCreator = new InputCreator<T>();
		//	InputFactory::GetInstance().RegisterInputType(id, pCreator);
		//}

		// Register an input creation object against an identifier.
		void	RegisterInputType(const Identifier& typeId, const shared_ptr<InputCreatorBase>& pCreator);

		// Create an instance (if we have a creator registered for it)
		shared_ptr<Input>	Create( PlayerID plyr, const DataRecord& data );

		// Data access helpers
		void	GetInputTypeIDs(std::vector<Identifier>& outTypes) const;

	private:
		InputCreatorBase*	FindCreator(const Identifier& id);
		void				Cleanup();

		typedef std::pair<Identifier, shared_ptr<InputCreatorBase> > IdCreatorPair;
		typedef std::vector<IdCreatorPair> IdCreatorMap;
		IdCreatorMap	m_CreatorMap;
	};

} // end namespace HInput

#endif // _InputFactory_h__