// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _InputListener_h__
#define _InputListener_h__

#include "Identifier.h"
#include "Enums.h"
#include <list>

namespace HInput
{
	// Fwd decs
	class Input;

	/*
		This encapsulates the data passed in the notification callback
	*/
	struct InputListenerCallbackData
	{
		InputListenerCallbackData();
		InputListenerCallbackData(const Input& input);
			
		PlayerID	Player;
		Identifier	InputID;
		bool		WasAvailable;
		bool		WasSatisfied;
		float		Value;
	};

	/*
		The InputListener interface allows client classes to subscribe
		to inputs. There is no direct connection to the Input itself,
		so the listener is safe if the Input is deleted, for example.

		The listener c'tor and d'tor automatically handle registration 
		with the InputManager.

		Instead, the subscriptions are held in the InputManager, which
		incurs a slight runtime overhead but which i feel is safer.

		It is worth noting that the client does not have to use this 
		'push' method: InputManager supports polling through WasSatisfied(),
		etc, if preferred.

		OnNotify() is called when the manager detects that the 'satisfied'
		value of an input has changed.
	*/
	class InputListener
	{
	public:
		InputListener();
		virtual ~InputListener(); // Just in case this is ever deleted through a base ptr

		enum SubscriptionType
		{
			SubscriptionType_OnlyWhenChanged = 0,
			SubscriptionType_EveryFrameWhenSatisfied,
			SubscriptionType_EveryFrameWhenUnsatisfied
		};

		virtual void	OnNotify(const InputListenerCallbackData& data) = 0;

		bool			IsListeningFor(const Identifier& inputID, SubscriptionType& outSubType ) const;
		void			AddInputSubscription(const Identifier& inputID, SubscriptionType type = SubscriptionType_OnlyWhenChanged);
		void			ClearInputSubscriptions();

	private:
		typedef std::pair<Identifier, SubscriptionType> IDTypePair;
		std::list<IDTypePair>	m_InputSubscriptions;
	};

} // end namespace HInput

#endif // _InputListener_h__