// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _InputManager_h__
#define _InputManager_h__

#include <string>
#include <vector>
#include <list>
#include "Enums.h"
#include "Input.h"
#include "SmartPtr.h"

namespace HInput
{
	// Fwd decs
	class TimeSource;
	class Identifier;
	class InputCreatorBase;
	class InputListener;
	class InputDeviceService;
	class DeviceServiceManager;
	class InputFactory;
	class MetaDataStore;
	class DataStore;
	class PluginManager;

	/*
		<<Singleton>>
		This class is the core of the HInput system, and the 
		client's main point of contact.

		Clients must call Initialize(), Update() and Shutdown() at 
		appropriate times in their game code.

		Clients are required to provide a TimeSource.

		Clients may (optionally) provide a logging callback.
	*/
	class InputManager
	{
	//////////////////////////////////////////////////////////////////////////
	// Logging stuff
	public:
		// Typedef a logging callback - it is assumed that the client will add a newline
		typedef void (*LogCallback)(const std::string&);
		void SetLogCallback(LogCallback pCallback) { m_pLogCallback = pCallback; }
		void WriteLogMsg(const std::string& msg);
		static void WriteLogMsg(const char* pFmt, ...);
	private:
		LogCallback m_pLogCallback;
	//////////////////////////////////////////////////////////////////////////

	private:
		InputManager();
		InputManager(const InputManager&); // Not implemented
		const InputManager& operator=(const InputManager&); // Not implemented
		~InputManager();

		enum LastState{ LastState_Unknown, LastState_Unsatisfied, LastState_Satisfied };
	
	public:
		static InputManager& GetInstance();
		
		// Main system life-cycle methods
		void		Initialize(TimeSource& rTimeSource, LogCallback pLogCallback = NULL);
		void		Shutdown();
		void		Update();

		// System query methods
		bool		WasInputAvailable(const Identifier& inputID, PlayerID plyrID = HInput::Player_One);
		bool		WasInputSatisfied(const Identifier& inputID, PlayerID plyrID = HInput::Player_One);
		float		GetInputValue    (const Identifier& inputID, PlayerID plyrID = HInput::Player_One);
		bool		DoesInputExist	 (const Identifier& inputID, PlayerID plyrID = HInput::Player_One);
		bool		IsPlayerActive   (PlayerID plyr);
		void		DumpDebugData	 (std::list<std::string>& outData) const;

		// System configuration methods
		void		SetPlayerActive			(PlayerID plyr, bool bActive);
		void		LoadInputSetForPlayer	(const Identifier& inputSetID, PlayerID plyrID = HInput::Player_One);
		void		ClearAllInputsForPlayer	(PlayerID plyr);
		void		SetupInputServices_Win32Keyboard();
		void		SetupInputServices_XInput();

		// Misc helpers
		const TimeSource&	GetTimeSource	() const;
		static bool			CheckValidPlayer(PlayerID plyr);
		const std::string&	GetFileDirectory() const;
		const std::string&	GetPluginDirectory() const;

		// Access to system components (former singletons)
		DeviceServiceManager*	GetDeviceServiceManager()	const { return m_pDeviceServiceManager; }
		InputFactory*			GetInputFactory()			const { return m_pInputFactory; }
		DataStore*				GetDataStore()				const { return m_pDataStore; }
		MetaDataStore*			GetMetaDataStore()			const { return m_pMetaDataStore; }

	private:

		// Listener methods
		friend class InputListener;
		void		RegisterListener(InputListener* pListener);
		void		RemoveListener  (InputListener* pListener);

		Input*		FindInput(const Identifier& inputID, PlayerID plyrID = HInput::Player_One);

		void		InitArrays();
		void		Cleanup();
		void		UpdateListeners(Input* pInput, LastState prevState);

		bool		m_bInitialized;
		TimeSource*	m_pTimeSource;

		typedef std::vector< std::pair< shared_ptr<Input>, LastState> > InputGroup;
		std::vector<InputGroup>			m_PlayerInputs;
		std::vector<bool>				m_PlayerActiveStates;
		std::list<InputListener*>		m_pListeners;

		LastState	GetState(Input* pInput) const;

		// Moved a bunch of singletons in here
		DeviceServiceManager*	m_pDeviceServiceManager;
		InputFactory*			m_pInputFactory;
		DataStore*				m_pDataStore;
		MetaDataStore*			m_pMetaDataStore;
		
		// Internal plugin manager
		PluginManager*			m_pPluginManager;
	};

	// More convenient access
	InputManager& GetManager();

} // end namespace HInput

#endif // _InputManager_h__
