// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _MetaDataStore_h__
#define _MetaDataStore_h__

#include "Identifier.h"
#include <string>
#include <list>
#include "Misc.h"

namespace HInput
{

	namespace InputMetaDataFlags
	{
		const unsigned UsesContext1		= 1 << 0;	// The input makes use of the context 1 variable
		const unsigned UsesContext2		= 1 << 1;	// The input makes use of the context 2 variable
		const unsigned UsesContextFloat	= 1 << 2;	// The input makes use of the context float variable

	} // end namespace InputMetaDataFlags


	/*
		A possible value for a context field, with a label
	*/
	struct InputContextOption 
	{
		InputContextOption()
			: Label("Not set")
			, Value(0)
		{}

		InputContextOption(const std::string& label, unsigned val)
			: Label(label)
			, Value(val)
		{}

		std::string		Label;
		unsigned		Value;
	};


	/*
		This class encapsulates details of how the input should be
		used - this is useful for tools that want to offer drop down
		choices, for example.
	*/
	class InputMetaData
	{
	public:
		typedef std::list<InputContextOption> OptionList;

		InputMetaData();
		InputMetaData(const InputMetaData& other);
		const InputMetaData& operator= (const InputMetaData& other);
		~InputMetaData();

		const Identifier&	GetTypeID() const { return m_TypeID; }
		void				SetTypeID(const Identifier& val) { m_TypeID = val; }

		bool	UsesContext1()	   const { return ((m_Flags & InputMetaDataFlags::UsesContext1) != 0); }
		bool	UsesContext2()	   const { return ((m_Flags & InputMetaDataFlags::UsesContext2) != 0); }
		bool	UsesContextFloat() const { return ((m_Flags & InputMetaDataFlags::UsesContextFloat) != 0); }

		void	SetFlag(unsigned flag) { m_Flags = m_Flags | flag; }
		void	ClrFlag(unsigned flag) { m_Flags = m_Flags & (~flag); }

		float	GetMinFloat		() const { return m_MinFloat; }
		float	GetMaxFloat		() const { return m_MaxFloat; }
		float	GetFloatStep	() const { return m_FloatStep; }
		float	GetDefaultFloat	() const { return m_DefaultFloat; }

		void	SetMinFloat		(const float& val) { m_MinFloat = val; }
		void	SetMaxFloat		(const float& val) { m_MaxFloat = val; }
		void	SetFloatStep	(const float& val) { m_FloatStep = val; }
		void	SetDefaultFloat	(const float& val) { m_DefaultFloat = val; }

		void	SetFloatData	(float def, float min, float max, float step) { m_DefaultFloat = def; m_MinFloat = min; m_MaxFloat = max; m_FloatStep = step;  }

		OptionList& OptionList1() { return m_OptionList1; }
		OptionList& OptionList2() { return m_OptionList2; }

	private:
		Identifier	m_TypeID;
		unsigned	m_Flags;

		float		m_MinFloat;
		float		m_MaxFloat;
		float		m_FloatStep;
		float		m_DefaultFloat;

		OptionList	m_OptionList1;
		OptionList	m_OptionList2;
	};


	/*
		Stores all the metadata for each input type, which is registered
		here by InputFactory::RegisterInputType
	*/
	class MetaDataStore : private Misc::Uncopyable
	{
	private:
		MetaDataStore();
		
		// InputManager is our friend so it can create us
		friend class InputManager;
	
		// InputFactory is our friend so it can register data with us
		friend class InputFactory;

	public:
		~MetaDataStore();
		
		bool	GetMetaDataForType(const Identifier& type, InputMetaData& outData) const;

	private:
		void						AddMetaData(const InputMetaData& data);
		std::list<InputMetaData>	m_MetaData;
	};


} // end namespace HInput

#endif // _MetaDataStore_h__