#ifndef __Misc_h__
#define __Misc_h__

// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#include <string>
#include <vector>

namespace HInput
{
	namespace Misc
	{
		template<typename T>
		void Clamp(T& val, const T min, const T max)
		{
			if (val < min) val = min;
			else if (val > max) val = max;
		}

		template<typename T>
		T Limit(const T val, const T min, const T max)
		{
			if (val < min) return min;
			else if (val > max) return max;
			else return val;
		}

		// Takes ptr to first char of string and ptr to wide char buffer of size
		// buffSz, which is where the result will be written if we return true.
		bool StringToWideString(const char* pSrc, wchar_t* pWideBuffer, size_t buffSz);
		void StringToWideString(const std::string& src, std::wstring& dst);
		void WideStringToString(const std::wstring& src, std::string& dst);


		/*
			Mixin for easily making classes uncopyable
			Ref: Scott Meyers, Effective C++ (3rd ed) p.39
		*/
		class Uncopyable
		{
		protected:
			// allow construction and destruction of derived objects...
			Uncopyable(){}
			~Uncopyable(){}
			
		private:
			// ...but prevent copying
			Uncopyable(const Uncopyable&);
			Uncopyable& operator=(const Uncopyable&);
		};


		/*
			A container of pointers that will be deleted when the 
			container is destroyed
		*/
		template<typename PointedTo, typename Container = std::vector<PointedTo*>>
		class SmartContainer : private Uncopyable
		{
		public:
			SmartContainer() {}
			~SmartContainer() { DeleteContents(); }
			
			// Returns a reference to the container. This is not
			// encapsulated since the caller must know its type anyway
			// to instantiate the template, and this way is more flexible.
			// The point of this wrapper is just to ensure deletion of
			// resources. 
			Container&			GetContainer() { return m_Container; }
			const Container&	GetContainer() const { return m_Container; }

			// Only pass pointers to memory dynamically allocated with
			// operator new. The pointer will be deleted automatically,
			// or when DeleteContents() is called.
			// DO NOT PASS POINTERS TO ARRAYS
			bool Add(PointedTo* ptr)
			{
				if (!ptr)
					return false;
				
				m_Container.push_back(ptr);
				return true;
			}

			// Find, delete and remove the pointer
			bool Remove(PointedTo* ptr)
			{
				if (!ptr)
					return false;

				for (Container::iterator it = m_Container.begin();
					it != m_Container.end(); ++it)
				{
					if (*it == ptr)
					{
						delete (*it);
						m_Container.erase(it);
						return true;
					}
				}
				return false;
			}
			
			// Call 'delete' on all pointers and clear the container
			void DeleteContents()
			{
				for (Container::iterator it = m_Container.begin();
					 it != m_Container.end(); ++it)
				{
					delete (*it);
				}
				m_Container.clear();
			}

		private:
			Container	m_Container;
		};

	} // Misc
}// HInput

#endif // __Misc_h__