// Copyright  2010 Henry Golding
// This file is part of the "HInput" input abstraction layer.
// For conditions of distribution and use, see copyright notice in HInput.h

#ifndef _TimeSource_HINPUT_h__
#define _TimeSource_HINPUT_h__

namespace HInput
{

	/*
		The TimeSource interface allows clients to supply a custom
		time source to the HInput::InputManager singleton, so the 
		library can be independent of platform timings.

		Client must implement a class derived from TimeSource and 
		tell the system about it e.g.

		MyTimeSource ts; // Derived from HInput::TimeSource
		HInput::InputManager.GetInstance().SetTimeSource( &ts );

		Ensure 'ts' remains in scope until after 
		HInput::InputManager::Shutdown() is called.
	*/
	class TimeSource
	{
	public:
		TimeSource(){ /**/ }
		virtual ~TimeSource(){ /**/ }

		// Get the time in seconds since the start of the game
		virtual float		GetTime() const = 0;

		// Get the duration in seconds of the last frame
		virtual float		GetFrameDuration() const = 0;

		// Get the sequence number of the current frame (used to avoid double updates)
		virtual unsigned	GetFrameNum() const = 0;
	};

	// This will not work like a proper clock, but it will advance and return semi-sensible values
	class DummyTimeSource : public TimeSource
	{
	public:
		DummyTimeSource(){ /**/ }
		virtual ~DummyTimeSource(){ /**/ }

		float		GetTime() const { static float time = 0.0f; time += (1.0f/30.0f); return time; }
		float		GetFrameDuration() const { return 1.0f / 30.0f; }
		unsigned	GetFrameNum() const { unsigned seq = 0; return ++seq; }
	};

} // end namespace HInput

#endif // _TimeSource_HINPUT_h__